import datetime as pydatetime
import urllib.parse
from typing import cast

import geopandas as gpd
import pystac
import pytest
from pystac.utils import datetime_to_str

from stac_generator._types import CsvMediaType
from stac_generator.base.generator import VectorGenerator
from stac_generator.csv.generator import read_csv
from stac_generator.csv.schema import CsvConfig
from tests import REMOTE_FIXTURE_URL

ALL_COLUMNS = {
    "latitude",
    "longitude",
    "elevation",
    "station",
    "YYYY-MM-DD",
    "daily_rain",
    "max_temp",
    "min_temp",
    "radiation",
    "mslp",
}
X = "latitude"
Y = "longitude"
Z = "elevation"
T = "YYYY-MM-DD"
EPSG = 7843  # GDA2020
DATE_FORMAT = "%Y-%M-%d"

MULTIPOINT_NO_DATE = "unit_tests/point_test/multi_point_no_date.csv"
MULTIPOINT_WITH_DATE = "unit_tests/point_test/multi_point_with_date.csv"
SINGLE_POINT_WITH_DATE = "unit_tests/point_test/single_point_with_date.csv"
SINGLE_POINT_NO_DATE = "unit_tests/point_test/single_point_no_date.csv"

MULTIPOINT_NO_DATE_ASSET = pystac.Asset(
    href=urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), MULTIPOINT_NO_DATE),
    roles=["data"],
    media_type=CsvMediaType,
)
MULTIPOINT_WITH_DATE_ASSET = pystac.Asset(
    href=urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), MULTIPOINT_WITH_DATE),
    roles=["data"],
    media_type=CsvMediaType,
)
SINGLE_POINT_WITH_DATE_ASSET = pystac.Asset(
    href=urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_WITH_DATE),
    roles=["data"],
    media_type=CsvMediaType,
)
SINGLE_POINT_NO_DATE_ASSET = pystac.Asset(
    href=urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_NO_DATE),
    roles=["data"],
    media_type=CsvMediaType,
)

SINGLE_POINT_GEOMETRY = {"type": "Point", "coordinates": [-34.9524, 138.5196]}
MULTIPOINT_GEOMETRY = {
    "type": "MultiPoint",
    "coordinates": [
        [-34.9524, 138.5196],
        [-34.9624, 138.5296],
        [-34.9724, 138.5396],
        [-34.9824, 138.5496],
    ],
}

START_DATE_DUMMY = pydatetime.datetime(2011, 1, 1, 12, 4, 5, tzinfo=pydatetime.UTC)
END_DATE_DUMMY = pydatetime.datetime(2011, 2, 1, 12, 4, 5, tzinfo=pydatetime.UTC)


@pytest.fixture(scope="module")
def multipoint_with_date_df() -> gpd.GeoDataFrame:
    return read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), MULTIPOINT_WITH_DATE),
        X,
        Y,
        T_coord=T,
        epsg=EPSG,
        date_format=DATE_FORMAT,
    )


@pytest.fixture(scope="module")
def multipoint_no_date_df() -> gpd.GeoDataFrame:
    return read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), MULTIPOINT_NO_DATE), X, Y, epsg=EPSG
    )


@pytest.fixture(scope="module")
def single_point_with_date_df() -> gpd.GeoDataFrame:
    return read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_WITH_DATE),
        X,
        Y,
        epsg=EPSG,
        T_coord=T,
        date_format=DATE_FORMAT,
    )


@pytest.fixture(scope="module")
def single_point_no_date_df() -> gpd.GeoDataFrame:
    return read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_NO_DATE), X, Y, epsg=EPSG
    )


def test_read_csv_given_no_args_read_all_columns() -> None:
    df = read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_WITH_DATE), X, Y, epsg=EPSG
    )
    expected = set(ALL_COLUMNS) | {"geometry"}
    assert set(df.columns) == expected


@pytest.mark.parametrize(
    "z_col, t_col, columns",
    [
        (Z, T, ["max_temp", "min_temp"]),
        (Z, None, ["max_temp", "min_temp"]),
        (None, T, ["max_temp", "min_temp"]),
        (None, None, ["max_temp"]),
    ],
)
def test_read_csv_given_selected_columns_read_selected_columns(
    z_col: str | None,
    t_col: str | None,
    columns: list[str],
) -> None:
    df = read_csv(
        urllib.parse.urljoin(str(REMOTE_FIXTURE_URL), SINGLE_POINT_WITH_DATE),
        X,
        Y,
        epsg=EPSG,
        Z_coord=z_col,
        T_coord=t_col,
        columns=columns,
    )
    expected_columns = {X, Y, "geometry"}
    if z_col is not None:
        expected_columns.add(z_col)
    if t_col is not None:
        expected_columns.add(t_col)
    expected_columns = expected_columns | set(columns)
    assert set(df.columns) == expected_columns


@pytest.mark.parametrize(
    "datetime, start_datetime, end_datetime",
    [
        (None, START_DATE_DUMMY, END_DATE_DUMMY),
        (END_DATE_DUMMY, START_DATE_DUMMY, END_DATE_DUMMY),
        (END_DATE_DUMMY, None, None),
    ],
)
def test_calculate_temporal_extent_given_single_point_with_date_always_return_df_date(
    datetime: pydatetime.datetime | None,
    start_datetime: pydatetime.datetime | None,
    end_datetime: pydatetime.datetime | None,
    single_point_with_date_df: gpd.GeoDataFrame,
) -> None:
    actual = VectorGenerator.temporal_extent(
        single_point_with_date_df, T, datetime, start_datetime, end_datetime
    )
    expected = (
        single_point_with_date_df.loc[:, T].min(),
        single_point_with_date_df.loc[:, T].max(),
    )
    assert actual == expected


@pytest.mark.parametrize(
    "datetime, start_datetime, end_datetime",
    [
        (None, START_DATE_DUMMY, END_DATE_DUMMY),
        (END_DATE_DUMMY, START_DATE_DUMMY, END_DATE_DUMMY),
        (END_DATE_DUMMY, None, None),
    ],
)
def test_calculate_temporal_extent_given_multiple_points_with_date_always_return_df_date(
    datetime: pydatetime.datetime | None,
    start_datetime: pydatetime.datetime | None,
    end_datetime: pydatetime.datetime | None,
    multipoint_with_date_df: gpd.GeoDataFrame,
) -> None:
    actual = VectorGenerator.temporal_extent(
        multipoint_with_date_df, T, datetime, start_datetime, end_datetime
    )
    expected = (
        multipoint_with_date_df.loc[:, T].min(),
        multipoint_with_date_df.loc[:, T].max(),
    )
    assert actual == expected


@pytest.mark.parametrize(
    "datetime, start_datetime, end_datetime, expected",
    [
        (None, START_DATE_DUMMY, END_DATE_DUMMY, (START_DATE_DUMMY, END_DATE_DUMMY)),
        (END_DATE_DUMMY, START_DATE_DUMMY, END_DATE_DUMMY, (START_DATE_DUMMY, END_DATE_DUMMY)),
        (END_DATE_DUMMY, None, None, (END_DATE_DUMMY, END_DATE_DUMMY)),
    ],
)
def test_calculate_temporal_extent_given_single_point_no_date_return_date_arguments(
    datetime: pydatetime.datetime | None,
    start_datetime: pydatetime.datetime | None,
    end_datetime: pydatetime.datetime | None,
    expected: tuple[pydatetime.datetime, pydatetime.datetime],
    single_point_no_date_df: gpd.GeoDataFrame,
) -> None:
    actual = VectorGenerator.temporal_extent(
        single_point_no_date_df, None, datetime, start_datetime, end_datetime
    )
    assert actual == expected


@pytest.mark.parametrize(
    "datetime, start_datetime, end_datetime, expected",
    [
        (None, START_DATE_DUMMY, END_DATE_DUMMY, (START_DATE_DUMMY, END_DATE_DUMMY)),
        (END_DATE_DUMMY, START_DATE_DUMMY, END_DATE_DUMMY, (START_DATE_DUMMY, END_DATE_DUMMY)),
        (END_DATE_DUMMY, None, None, (END_DATE_DUMMY, END_DATE_DUMMY)),
    ],
)
def test_calculate_temporal_extent_given_multipoint_no_date_return_date_arguments(
    datetime: pydatetime.datetime | None,
    start_datetime: pydatetime.datetime | None,
    end_datetime: pydatetime.datetime | None,
    expected: tuple[pydatetime.datetime, pydatetime.datetime],
    multipoint_no_date_df: gpd.GeoDataFrame,
) -> None:
    actual = VectorGenerator.temporal_extent(
        multipoint_no_date_df, None, datetime, start_datetime, end_datetime
    )
    assert actual == expected


@pytest.mark.parametrize(
    "source_cfg, exp_datetime, exp_start_datetime, exp_end_datetime",
    [
        (
            CsvConfig(X=X, Y=Y, id="test_id", location="", datetime=END_DATE_DUMMY, gsd=None),
            END_DATE_DUMMY,
            END_DATE_DUMMY,
            END_DATE_DUMMY,
        ),
        (
            CsvConfig(
                X=X,
                Y=Y,
                id="test_id",
                location="",
                datetime=None,
                start_datetime=START_DATE_DUMMY,
                end_datetime=END_DATE_DUMMY,
                gsd=None,
            ),
            END_DATE_DUMMY,
            START_DATE_DUMMY,
            END_DATE_DUMMY,
        ),
    ],
)
def test_df_to_item_single_point_with_date_given_no_config_date_column_expect_date_from_config(
    source_cfg: CsvConfig,
    exp_datetime: pydatetime.datetime,
    exp_start_datetime: pydatetime.datetime,
    exp_end_datetime: pydatetime.datetime,
    single_point_with_date_df: gpd.GeoDataFrame,
) -> None:
    item = VectorGenerator.df_to_item(
        single_point_with_date_df,
        assets={"data": SINGLE_POINT_WITH_DATE_ASSET},
        source_cfg=source_cfg,
        properties={},
        time_col=source_cfg.T,
        epsg=source_cfg.epsg,
    )
    assert item.id == source_cfg.id
    assert item.datetime == exp_datetime
    assert item.properties["start_datetime"] == datetime_to_str(exp_start_datetime)
    assert item.properties["end_datetime"] == datetime_to_str(exp_end_datetime)
    assert item.assets == {"data": SINGLE_POINT_WITH_DATE_ASSET}
    assert item.geometry == SINGLE_POINT_GEOMETRY


@pytest.mark.parametrize(
    "source_cfg, exp_datetime, exp_start_datetime, exp_end_datetime",
    [
        (
            CsvConfig(X=X, Y=Y, id="test_id", location="", datetime=END_DATE_DUMMY, gsd=None),
            END_DATE_DUMMY,
            END_DATE_DUMMY,
            END_DATE_DUMMY,
        ),
        (
            CsvConfig(
                X=X,
                Y=Y,
                id="test_id",
                location="",
                datetime=None,
                start_datetime=START_DATE_DUMMY,
                end_datetime=END_DATE_DUMMY,
                gsd=None,
            ),
            END_DATE_DUMMY,
            START_DATE_DUMMY,
            END_DATE_DUMMY,
        ),
    ],
)
def test_df_to_item_single_point_no_date(
    source_cfg: CsvConfig,
    exp_datetime: pydatetime.datetime,
    exp_start_datetime: pydatetime.datetime,
    exp_end_datetime: pydatetime.datetime,
    single_point_no_date_df: gpd.GeoDataFrame,
) -> None:
    item = VectorGenerator.df_to_item(
        single_point_no_date_df,
        assets={"data": SINGLE_POINT_WITH_DATE_ASSET},
        source_cfg=source_cfg,
        properties={},
        time_col=source_cfg.T,
        epsg=source_cfg.epsg,
    )
    assert item.id == source_cfg.id
    assert item.datetime == exp_datetime
    assert item.properties["start_datetime"] == datetime_to_str(exp_start_datetime)
    assert item.properties["end_datetime"] == datetime_to_str(exp_end_datetime)
    assert item.assets == {"data": SINGLE_POINT_WITH_DATE_ASSET}
    assert item.geometry == SINGLE_POINT_GEOMETRY


@pytest.mark.parametrize(
    "source_cfg",
    [
        CsvConfig(X=X, Y=Y, T=T, id="test_id", location="", datetime=END_DATE_DUMMY, gsd=None),
        CsvConfig(
            X=X,
            Y=Y,
            T=T,
            id="test_id",
            location="",
            datetime=None,
            start_datetime=START_DATE_DUMMY,
            end_datetime=END_DATE_DUMMY,
            gsd=None,
        ),
    ],
)
def test_df_to_item_single_point_given_config_with_date_column_expect_date_from_data(
    source_cfg: CsvConfig,
    single_point_with_date_df: gpd.GeoDataFrame,
) -> None:
    item = VectorGenerator.df_to_item(
        single_point_with_date_df,
        assets={"data": SINGLE_POINT_WITH_DATE_ASSET},
        source_cfg=source_cfg,
        properties={},
        time_col=source_cfg.T,
        epsg=source_cfg.epsg,
    )
    min_date, max_date = VectorGenerator.temporal_extent(single_point_with_date_df, T)
    assert item.id == source_cfg.id
    assert item.datetime == max_date
    assert item.properties["start_datetime"] == datetime_to_str(cast(pydatetime.datetime, min_date))
    assert item.properties["end_datetime"] == datetime_to_str(cast(pydatetime.datetime, max_date))
    assert item.assets == {"data": SINGLE_POINT_WITH_DATE_ASSET}
    assert item.geometry == SINGLE_POINT_GEOMETRY


@pytest.mark.parametrize(
    "source_cfg",
    [
        CsvConfig(X=X, Y=Y, T=T, id="test_id", location="", datetime=END_DATE_DUMMY, gsd=None),
        CsvConfig(
            X=X,
            Y=Y,
            T=T,
            id="test_id",
            location="",
            datetime=None,
            start_datetime=START_DATE_DUMMY,
            end_datetime=END_DATE_DUMMY,
            gsd=None,
        ),
    ],
)
def test_df_to_item_multipoint_with_date_given_config_with_date_column_expect_date_from_data(
    source_cfg: CsvConfig,
    multipoint_with_date_df: gpd.GeoDataFrame,
) -> None:
    item = VectorGenerator.df_to_item(
        multipoint_with_date_df,
        assets={"data": SINGLE_POINT_WITH_DATE_ASSET},
        source_cfg=source_cfg,
        properties={},
        time_col=source_cfg.T,
        epsg=source_cfg.epsg,
    )
    min_date, max_date = VectorGenerator.temporal_extent(multipoint_with_date_df, T)
    assert item.id == source_cfg.id
    assert item.datetime == max_date
    assert item.properties["start_datetime"] == datetime_to_str(cast(pydatetime.datetime, min_date))
    assert item.properties["end_datetime"] == datetime_to_str(cast(pydatetime.datetime, max_date))
    assert item.assets == {"data": SINGLE_POINT_WITH_DATE_ASSET}
    assert item.geometry == MULTIPOINT_GEOMETRY



import json
import logging
import urllib.parse
from pathlib import Path
from typing import Any, cast

import httpx
import pandas as pd
import yaml

SUPPORTED_URI_SCHEMES = ["http", "https"]
logger = logging.getLogger(__name__)


def parse_href(base_url: str, collection_id: str, item_id: str | None = None) -> str:
    """Generate href for collection or item based on id

    :param base_url: path to catalog.
    :type base_url: str
    :param collection_id: collection id
    :type collection_id: str
    :param item_id: item id, defaults to None
    :type item_id: str | None, optional
    :return: uri to collection or item
    :rtype: str
    """
    if item_id:
        return urllib.parse.urljoin(base_url, f"{collection_id}/{item_id}")
    return urllib.parse.urljoin(base_url, f"{collection_id}")


def href_is_stac_api_endpoint(href: str) -> bool:
    """Check if href points to a resource behind a stac api

    :param href: path to resource
    :type href: str
    :return: local or non-local
    :rtype: bool
    """
    output = urllib.parse.urlsplit(href)
    return output.scheme == ""


def force_write_to_stac_api(url: str, id: str, json: dict[str, Any]) -> None:
    """Force write a json object to a stac api endpoint.
    This function will try sending a POST request and if a 409 error is encountered,
    try sending a PUT request. Other exceptions if occured will be raised

    :param url: path to stac resource for creation/update
    :type url: str
    :param json: json object
    :type json: dict[str, Any]
    """
    try:
        logger.debug(f"sending POST request to {url}")
        response = httpx.post(url=url, json=json)
        response.raise_for_status()
    except httpx.HTTPStatusError as err:
        if err.response.status_code == 409:
            logger.debug(f"sending PUT request to {url}")
            response = httpx.put(url=f"{url}/{id}", json=json)
            response.raise_for_status()
        else:
            raise err


def read_source_config(href: str) -> list[dict[str, Any]]:
    logger.debug(f"reading config file from {href}")
    if not href.endswith(("json", "yaml", "yml", "csv")):
        raise ValueError(
            "Unsupported extension. We currently allow json/yaml/csv files to be used as config"
        )
    if href.endswith(".csv"):
        df = pd.read_csv(href)
        return cast(list[dict[str, Any]], df.to_dict("records"))
    if not href.startswith(("http", "https")):
        with Path(href).open("r") as file:
            if href.endswith(("yaml", "yml")):
                result = yaml.safe_load(file)
            if href.endswith("json"):
                result = json.load(file)
    else:
        response = httpx.get(href, follow_redirects=True)
        response.raise_for_status()
        if href.endswith("json"):
            result = response.json()
        if href.endswith(("yaml", "yml")):
            result = yaml.safe_load(response.content.decode("utf-8"))

    if isinstance(result, dict):
        return [result]
    if isinstance(result, list):
        return result
    raise ValueError(f"Expects config to be read as a list of dictionary. Provided: {type(result)}")



from stac_generator.base.schema import SourceConfig


class VectorConfig(SourceConfig):
    """Extended source config with EPSG code."""

    epsg: int = 4326
    """EPSG code for checking against EPSG code of the vector data"""



from __future__ import annotations

import datetime
import math
from typing import Any, TypeVar

from httpx._types import (
    RequestData,  # noqa: TCH002
)
from pydantic import model_validator
from stac_pydantic.shared import StacCommonMetadata as _StacCommonMetaData

from stac_generator._types import (  # noqa: TCH001
    CookieTypes,
    HeaderTypes,
    HTTPMethod,
    QueryParamTypes,
    RequestContent,
)

T = TypeVar("T", bound="SourceConfig")


class StacCommonMetadata(_StacCommonMetaData):
    """Stac Common Metadata. Automatically sets datetime values to
    current datetime if neither datetime nor start_datetime and end_datetime are provided
    """

    @model_validator(mode="before")
    @classmethod
    def set_datetime(self, data: Any) -> Any:
        if isinstance(data, dict):
            if "datetime" not in data or (
                data["datetime"] is not None
                and not isinstance(data["datetime"], datetime.datetime)
                and math.isnan(data["datetime"])
            ):
                data["datetime"] = None
            if (
                (data["datetime"] is None)
                and ("start_datetime" not in data or data["start_datetime"] is None)
                and ("end_datetime" not in data or data["end_datetime"] is None)
            ):
                now = datetime.datetime.now(datetime.UTC)
                data["datetime"] = now
                data["start_datetime"] = now
                data["end_datetime"] = now
        return data


class StacCollectionConfig(StacCommonMetadata):
    """Contains parameters to pass to Collection constructor. Also contains other metadata

    This config provides additional information that can not be derived from source file, which includes
    <a href="https://github.com/radiantearth/stac-spec/blob/master/commons/common-metadata.md">Stac Common Metadata</a>
    and other descriptive information such as the id of the new entity
    """

    # Stac Information
    id: str
    """Collection id"""
    title: str = "Auto-generated"
    """Collection title"""
    description: str = "Auto-generated Stac Collection"
    """Collection description"""


class StacItemConfig(StacCommonMetadata):
    """Contains parameters to pass to Item constructor.

    This config provides additional information that can not be derived from source file, which includes
    <a href="https://github.com/radiantearth/stac-spec/blob/master/commons/common-metadata.md">Stac Common Metadata</a>
    and other descriptive information such as the id of the new entity
    """

    id: str
    """Item id - doubles as prefix if there are multiple items extracted from the source file"""
    description: str = "Auto-generated Stac Item"
    """Item description"""


class SourceConfig(StacItemConfig):
    """Base source config that should be subclassed for different file extensions.

    Source files contain raw spatial information (i.e. geotiff, shp, csv) from which
    some Stac metadata can be derived. SourceConfig describes:

    - The access mechanisms for the source file: stored on local disk, or hosted somewhere behind an api endpoint. If the source
    file must be accessed through an endpoint, users can provide additional HTTP information that forms the HTTP request to the host server.
    - Processing information that are unique for the source type. Users should inherit `SourceConfig` for file extensions
    currently unsupported.
    - Additional Stac Metadata from `StacConfig`
    """

    location: str
    """Asset's href.
    """
    extension: str | None = None
    """Explicit file extension specification. If the file is stored behind an api endpoint, the field `extension` must be provided"""
    # HTTP Parameters
    method: HTTPMethod | None = "GET"
    """HTTPMethod to acquire the file from `location`"""
    params: QueryParamTypes | None = None
    """HTTP query params for getting file from `location`"""
    headers: HeaderTypes | None = None
    """HTTP query headers for getting file from `location`"""
    cookies: CookieTypes | None = None
    """HTTP query cookies for getting file from `location`"""
    content: RequestContent | None = None
    """HTTP query body content for getting file from `location`"""
    data: RequestData | None = None
    """HTTP query body content for getting file from `location`"""
    json_body: Any = None
    """HTTP query body content for getting file from `location`"""

    @property
    def source_extension(self) -> str:
        if self.extension:
            return self.extension
        return self.location.split(".")[-1]



import urllib.parse

import httpx
import pytest

from stac_generator.base.generator import CollectionGenerator
from stac_generator.base.schema import StacCollectionConfig
from stac_generator.base.utils import read_source_config
from stac_generator.csv.generator import CsvGenerator
from stac_generator.csv.schema import CsvConfig
from tests import REMOTE_FIXTURE_URL

REMOTE_CONFIG_JSON = urllib.parse.urljoin(
    str(REMOTE_FIXTURE_URL), "Adelaide/point/point_data_config.json"
)

REMOTE_CONFIG_CSV = urllib.parse.urljoin(
    str(REMOTE_FIXTURE_URL), "Adelaide/point/point_data_config.csv"
)

REMOTE_GENERATED_DIR = urllib.parse.urljoin(
    str(REMOTE_FIXTURE_URL), "Adelaide/point/stac_generated/"
)


JSON_CONFIGS = read_source_config(REMOTE_CONFIG_JSON)
CSV_CONFIGS = read_source_config(REMOTE_CONFIG_CSV)


@pytest.fixture(scope="module")
def json_csv_generator() -> CsvGenerator:
    return CsvGenerator(JSON_CONFIGS)


@pytest.fixture(scope="module")
def csv_generator() -> CsvGenerator:
    return CsvGenerator(CSV_CONFIGS)


@pytest.fixture(scope="module")
def collection_generator(csv_generator: CsvGenerator) -> CollectionGenerator:
    return CollectionGenerator(
        StacCollectionConfig(id="point_data", datetime=None, gsd=None), generators=[csv_generator]
    )


@pytest.mark.parametrize("item_idx", range(len(JSON_CONFIGS)))
def test_generator_given_item_expects_matched_generated_item(
    item_idx: int, json_csv_generator: CsvGenerator
) -> None:
    config = JSON_CONFIGS[item_idx]
    expected_path = urllib.parse.urljoin(config["location"], f"stac_generated/{config['id']}.json")
    data = httpx.get(expected_path)
    expected = data.json()
    actual = json_csv_generator.create_item_from_config(CsvConfig(**config)).to_dict()
    assert expected["id"] == actual["id"]
    assert expected["bbox"] == actual["bbox"]
    assert expected["properties"] == actual["properties"]
    assert expected["assets"] == actual["assets"]
    assert expected["geometry"] == actual["geometry"]


@pytest.mark.parametrize("item_idx", range(len(CSV_CONFIGS)))
def test_generator_given_item_expects_matched_generated_item_csv_config_version(
    item_idx: int, csv_generator: CsvGenerator
) -> None:
    config = JSON_CONFIGS[item_idx]
    expected_path = urllib.parse.urljoin(config["location"], f"stac_generated/{config['id']}.json")
    data = httpx.get(expected_path)
    expected = data.json()
    actual = csv_generator.create_item_from_config(CsvConfig(**config)).to_dict()
    assert expected["id"] == actual["id"]
    assert expected["bbox"] == actual["bbox"]
    assert expected["properties"] == actual["properties"]
    assert expected["assets"] == actual["assets"]
    assert expected["geometry"] == actual["geometry"]


def test_collection_generator(collection_generator: CollectionGenerator) -> None:
    actual = collection_generator.create_collection().to_dict()
    expected_path = urllib.parse.urljoin(REMOTE_GENERATED_DIR, "collection.json")
    data = httpx.get(expected_path)
    expected = data.json()
    assert actual["extent"] == expected["extent"]



import logging

from stac_generator.factory import StacGeneratorFactory

__all__ = ("StacGeneratorFactory",)

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)

for name in logging.root.manager.loggerDict:
    if not name.startswith(__name__):
        logging.getLogger(name).disabled = True
